<?php
/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2006 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: DaySpan.php 4346 2006-03-06 16:43:19Z andrew@m3.net $
*/

/**
 * A class which contains a span of days (start date and end date), to be used in statistics and reporting functions.
 *
 * @package    Max
 * @author     Scott Switzer <scott@switzer.org>
 * @TODO This should be a replacement for the DateRange class
 */
class DaySpan
{
    /* @var Date */
    var $_startDate;

    /* @var Date */
    var $_endDate;

    /**
     * PHP5-style constructor
     *
     * @param string an initial preset value based on a 'friendly' date span.
     */
    function __construct($presetValue = 'today')
    {
        $this->setSpanPresetValue($presetValue);
    }

    /**
     * PHP4-style constructor
     *
     * @param string an initial preset value based on a 'friendly' date span.
     */
    function DaySpan($presetValue = 'today')
    {
        $this->__construct($presetValue);
    }
    
    /**
     * A method to set the span according to one of the pre-defined 'friendly' dates.
     * The predefined values are:
     *     today, yesterday, this_week, last_week, last_7_days, this_month, last_maonth,
     *     this_month_remainder, next_month
     *
     * @param string $presetValue The preset value string.
     */
   function setSpanPresetValue($presetValue)
    {
        switch ($presetValue) {
            case 'today' :
                $dateStart = & new Date();
                $dateEnd   = & new Date();
                $this->setSpanDays($dateStart, $dateEnd);
                break;
            case 'yesterday' :
                $dateStart = & new Date(Date_Calc::prevDay());
                $dateEnd   = & new Date(Date_Calc::prevDay());
                $this->setSpanDays($dateStart, $dateEnd);
                break;
            case 'last_7_days' :
                $dateStart = & new Date();
                $dateStart->subtractSpan(new Date_Span('7, 0, 0, 0'));
                $dateEnd   = & new Date();
                $dateEnd->subtractSpan(new Date_Span('1, 0, 0, 0'));
                $this->setSpanDays($dateStart, $dateEnd);
                break;
            case 'this_week' :
                $dateStart = & new Date(Date_Calc::beginOfWeek());
                $dateEnd   = & new Date();
                $this->setSpanDays($dateStart, $dateEnd);
                break;
            case 'last_week' :
                $dateStart = & new Date(Date_Calc::beginOfPrevWeek());
                $dateEnd   = & new Date(Date_Calc::beginOfWeek());
                $dateEnd->subtractSpan(new Date_Span('1, 0, 0, 0'));
                $this->setSpanDays($dateStart, $dateEnd);
                break;
            case 'this_month' : 
                $dateStart = & new Date(Date_Calc::beginOfMonth());
                $dateEnd   = & new Date(Date_Calc::beginOfNextMonth());
                $dateEnd->subtractSpan(new Date_Span('1, 0, 0, 0'));
                $this->setSpanDays($dateStart, $dateEnd);
                break;
            case 'this_month_remainder' : 
                $dateStart = & new Date();
                $dateEnd   = & new Date(Date_Calc::beginOfNextMonth());
                $dateEnd->subtractSpan(new Date_Span('1, 0, 0, 0'));
                $this->setSpanDays($dateStart, $dateEnd);
                break;
            case 'next_month' :
                $dateStart = & new Date(Date_Calc::beginOfNextMonth());
                $dateEnd   = & new Date(Date_Calc::EndOfNextMonth());
                $dateEnd->subtractSpan(new Date_Span('1, 0, 0, 0'));
                $this->setSpanDays($dateStart, $dateEnd);
                break;
            case 'last_month' :
                $dateStart = & new Date(Date_Calc::beginOfPrevMonth());
                $dateEnd   = & new Date(Date_Calc::beginOfMonth());
                $dateEnd->subtractSpan(new Date_Span("1, 0, 0, 0"));
                $this->setSpanDays($dateStart, $dateEnd);
                break;
        }
    }

    /**
     * A method to set the span according to specific dates.
     *
     * @param Date $oStartDate The start date.
     * @param Date $oEndDate The end date.
     */
    function setSpanDays($oStartDate, $oEndDate)
    {
        $this->_startDate = $oStartDate;
        $this->_endDate = $oEndDate;
    }
    
    /**
     * The start day of the DaySpan.
     * @return Date A date object representing the start of the span
     */
    function getStartDate()
    {
        return $this->_startDate;
    }

    /**
     * The end day of the DaySpan.
     * @return Date A date object representing the end of the span
     */
    function getEndDate()
    {
        return $this->_endDate;
    }

    /**
     * A string representation of the start day of the DaySpan.
     * @param string $format The format of the string (see format rules in Pear::Date->format)
     * @return Date A date object representing the start of the span
     */
    function getStartDateString($format = '%Y-%m-%d')
    {
        
        return $this->_startDate->format($format);
    }

    /**
     * A string representation of the end day of the DaySpan.
     * @param string $format The format of the string (see format rules in Pear::Date->format)
     * @return Date A date object representing the end of the span
     */
    function getEndDateString($format = '%Y-%m-%d')
    {
        return $this->_endDate->format($format);
    }
    
    /**
     * A method to return the number of days in the span (including the start and end date)
     * @return integer The number of days covered by the span (inclusive).
     */
    function getDaysInSpan()
    {
        $startDate = $this->getStartDate();
        $endDate   = $this->getEndDate();
        
        $days = Date_Calc::dateDiff(
            $startDate->getDay(),
            $startDate->getMonth(),
            $startDate->getYear(),
            $endDate->getDay(),
            $endDate->getMonth(),
            $endDate->getYear()
        );
        
        return $days;
    }
    
    /**
     * A method to return the number of instances of each day of week (0=Sunday, 1=Monday, etc.)
     * @return array The number of instances of each day of week (0=Sunday, 1=Monday, etc.)
     */
    function getDayOfWeekCount()
    {
        $aDayOfWeek = array(
            0 => 0,
            1 => 0,
            2 => 0,
            3 => 0,
            4 => 0,
            5 => 0,
            6 => 0,
        );
        
        $startDay = $this->getStartDate();
        $endDay = $this->getEndDate();

        $activeDay = new Date($startDay);
        
        while (!$activeDay->after($endDay))
        {
            $dayOfWeek = $activeDay->getDayOfWeek();
            $aDayOfWeek[$dayOfWeek]++;
            $activeDay->addSpan(new Date_Span('1, 0, 0, 0'));
        }
        return $aDayOfWeek;
        
    }
}

?>