<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2006 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: Maintenance.php 4614 2006-04-05 17:04:20Z andrew@m3.net $
*/

require_once MAX_PATH . '/lib/max/core/ServiceLocator.php';
require_once MAX_PATH . '/lib/max/Dal/Maintenance.php';

/**
 * A class library for providing common methods for the maintenance scripts.
 *
 * @package    MaxMaintenance
 * @author     Andrew Hill <andrew@m3.net>
 */
class Maintenance
{

    /**
     * The constructor method for the class library.
     *
     * The class library needs access to the maintenance statistics
     * data access layer, so instantiate and register the layer
     * in the constructor.
     */
    function Maintenance()
    {
        $oServiceLocator = &ServiceLocator::instance();
        $dbh = &$oServiceLocator->get('MAX_Dal_Maintenance');
        if (!$dbh) {
            $dbh = MAX_Dal_Maintenance::singleton();
            $oServiceLocator->register('MAX_Dal_Maintenance', $dbh);
        }
    }

    /**
     * A class to test if an operation interval value is valid.
     *
     * @param integer $oi The operation interval value in minutes.
     * @param mixed True if the operation interval value is valid, a {@link PEAR_Error}
     *              object with error type MAX_ERROR_INVALIDOPERATIONINT otherwise.
     */
    function checkOperationIntervalValue($oi)
    {
        if ($oi < 1) {
            // Operation interval must be at least every minute
            $error = 'The operation interval of ' . $oi . ' is invalud';
            return MAX::raiseError($error, MAX_ERROR_INVALIDOPERATIONINT);
        } elseif ($oi < 60) {
            // Operation interval is more often than once an hour
            if ((60 % $oi) != 0) {
                // Operation interval must be a factor of 60 minutes
                $error = 'The operation interval of ' . $oi . ' is invalud';
                return MAX::raiseError($error, MAX_ERROR_INVALIDOPERATIONINT);
            }
        } elseif ($oi > 60) {
            // Operation interval is less often than once an hour
            if (($oi % 60) != 0) {
                // Operation interval must be a multiple of 60 minutes
                $error = 'The operation interval of ' . $oi . ' is invalud';
                return MAX::raiseError($error, MAX_ERROR_INVALIDOPERATIONINT);
            }
            if ($oi > 1440) {
                // Operation interval must not be more than one week (24 * 60 = 1440 minutes)
                $error = 'The operation interval of ' . $oi . ' is invalud';
                return MAX::raiseError($error, MAX_ERROR_INVALIDOPERATIONINT);
            }
        }
        return true;
    }

    /**
     * A static method to convert a date range (in the form of two Dates)
     * and return the operation interval ID represented by the range, or
     * false if the date range spans more than one operation interval ID.
     *
     * @static
     * @param Date $start The start date of the date range.
     * @param Date $end The end date of the date range.
     * @param integer $operation_interval Optional length of the operation interval
     *                                    in minutes. If not given, will use the
     *                                    currently defined operation interval.
     * @return integer|false The operation interval ID, or false if the date
     *                       range spans more than one operation interval.
     */
    function convertDateRangeToOperationIntervalID($start, $end, $operationInterval = 0)
    {
        if ($operationInterval < 1) {
            $operationInterval = Maintenance::getOperationInterval();
        }
        // Ensure the dates are in the same week
        if ($start->getYear() != $end->getYear()) {
            return false;
        }
        if ($start->getWeekOfYear() != $end->getWeekOfYear()) {
            return false;
        }
        // Find the operation interval ID of the start date
        $startID = Maintenance::convertDateToOperationIntervalID($start, $operationInterval);
        // Find the operation interval ID of the end date
        $endID = Maintenance::convertDateToOperationIntervalID($end, $operationInterval);
        // Compare the two IDs
        if ($startID != $endID) {
            return false;
        }
        return $startID;
    }

    /**
     * A static method to convert a Date object into the appropriate operation
     * interval ID (of the week).
     *
     * Works by finding out how many days, hours, minutes into the week the time is, and
     * then converting that into how many minutes into the week the time is. This value
     * is then divided by the interval length (in minutes) to determine how many intervals
     * into the week the time is.
     *
     * @static
     * @param Date $time The date to convert.
     * @param integer $operation_interval Optional length of the operation interval
     *                                    in minutes. If not given, will use the
     *                                    currently defined operation interval.
     * @return integer The operation interval ID that the time is in.
     */
    function convertDateToOperationIntervalID($date, $operationInterval = 0)
    {
        if ($operationInterval < 1) {
            $operationInterval = Maintenance::getOperationInterval();
        }
        $days = $date->getDayOfWeek();
        $hours = $date->getHour();
        $minutes = $date->getMinute();
        $totalMinutes = ($days * 24 * 60) + ($hours * 60) + $minutes;
        return(floor($totalMinutes / $operationInterval));
    }

    /**
     * A static method to convert a Date into an array containing the start
     * and end dates of the operation interval that the date is in.
     *
     * @static
     * @param Date $date The date to convert.
     * @param integer $operation_interval Optional length of the operation interval
     *                                    in minutes. If not given, will use the
     *                                    currently defined operation interval.
     * @return array An array of the start and end Dates of the operation interval.
     */
    function convertDateToOperationIntervalStartAndEndDates($date, $operationInterval = 0)
    {
        if ($operationInterval < 1) {
            $operationInterval = Maintenance::getOperationInterval();
        }
        // Get the date representing the start of the week
        $startOfWeek = new Date(Date_Calc::beginOfWeek($date->getDay(), $date->getMonth(), $date->getYear(), '%Y-%m-%d 00:00:00'));
        // Get the operation interval ID of the date
        $operationIntervalID = Maintenance::convertDateToOperationIntervalID($date, $operationInterval);
        // The start of the operation interval is the start of the week plus the
        // operation interval ID multiplied by the operation interval
        $start = new Date();
        $start->copy($startOfWeek);
        $start->addSeconds($operationIntervalID * $operationInterval * 60);
        // The end of the operation interval is the start of the week plus the
        // operation interval ID + 1 multiplied by the operation interval
        $end = new Date();
        $end->copy($start);
        $end->addSeconds(($operationInterval * 60) - 1);
        // Return the result
        return array($start, $end);
    }

    /**
     * A static method to find the previous operation interval ID.
     *
     * @static
     * @param integer $operationIntervalID The operation interval ID.
     * @param integer $operationInterval Optional length of the operation interval
     *                                   in minutes. If not given, will use the
     *                                   currently defined operation interval.
     * @return integer The previous operation interval ID.
     */
    function previousOperationIntervalID($operationIntervalID, $operationInterval = 0)
    {
        if ($operationInterval < 1) {
            $operationInterval = Maintenance::getOperationInterval();
        }
        if ($operationIntervalID == 0) {
            return ((MINUTES_PER_WEEK / $operationInterval) - 1);
        } else {
            return ($operationIntervalID - 1);
        }
    }

    /**
     * A static method to find the next operation interval ID.
     *
     * @static
     * @param integer $operationIntervalID The operation interval ID.
     * @param integer $operationInterval Optional length of the operation interval
     *                                   in minutes. If not given, will use the
     *                                   currently defined operation interval.
     * @return integer The next operation interval ID.
     */
    function nextOperationIntervalID($operationIntervalID, $operationInterval = 0)
    {
        if ($operationInterval < 1) {
            $operationInterval = Maintenance::getOperationInterval();
        }
        if ($operationIntervalID == ((MINUTES_PER_WEEK / $operationInterval) - 1)) {
            return 0;
        } else {
            return ($operationIntervalID + 1);
        }
    }

    /**
     * A static method to calculate and return a range of operation intervals.
     *
     * @static
     * @param array $operationIntervals A reference to an array that will be
     *                                  set to contain a series of arrays with
     *                                  the start and end Dates of the operation
     *                                  intervals in the range, indexed by
     *                                  the operation interval IDs.
     * @param Date $date A date in the FIRST operation interval of the range
     *                   to be calculated.
     * @param integer $intervals The number of operation intervals in the range,
     *                           inclusive of the first and last intervals.
     * @param integer $operationInterval Optional length of the operation interval
     *                                   in minutes. If not given, will use the
     *                                   currently defined operation interval.
     */
    function getOperationIntervalRange(&$operationIntervals, $date, $intervals, $operationInterval = 0)
    {
        if ($operationInterval < 1) {
            $operationInterval = Maintenance::getOperationInterval();
        }
        // Calculate the start and end date of the first operation interval
        list($start, $end) = Maintenance::convertDateToOperationIntervalStartAndEndDates($date, $operationInterval);
        // Calculate the operation interval ID
        $operationIntervalID = Maintenance::convertDateToOperationIntervalID($start, $operationInterval);
        // Store the operation interval
        $operationIntervals[$operationIntervalID] = array('start' => $start, 'end' => $end);
        for ($counter = 1; $counter < $intervals; $counter++) {
            // Calculate a date in the next operation interval
            $nextOperationIntervalDate = new Date();
            $nextOperationIntervalDate->copy($end);
            $nextOperationIntervalDate->addSeconds(1);
            // Calculate the start and end date of the next operation interval
            list($start, $end) = Maintenance::convertDateToOperationIntervalStartAndEndDates($nextOperationIntervalDate, $operationInterval);
            // Calculate the operation interval ID
            $operationIntervalID = Maintenance::convertDateToOperationIntervalID($start, $operationInterval);
            // Store the operation interval
            $operationIntervals[$operationIntervalID] = array('start' => $start, 'end' => $end);
        }
    }

    /**
     * A static method to ensure that a Date range is in the same hour.
     *
     * @static
     * @param Date $start The start date to check.
     * @param Date $end The end date to check.
     * @return boolean Returns true if the dates are in the same hour, false otherwise.
     */
    function checkDatesInSameHour($start, $end)
    {
        if ($start->getYear() != $end->getYear()) {
            return false;
        }
        if ($start->getMonth() != $end->getMonth()) {
            return false;
        }
        if ($start->getDay() != $end->getDay()) {
            return false;
        }
        if ($start->getHour() != $end->getHour()) {
            return false;
        }
        return true;
    }

    /**
     * A method to check that two Dates represent either the start and end
     * of an operation interval, if the operation interval is less than an
     * hour, or the start and end of an hour otherwise.
     *
     * @param Date $start The interval start date.
     * @param Date $end The interval end date.
     * @param integer $operationInterval The operation interval in minutes.
     * @return bool Returns true if the dates are correct interval
     *              start/end dates, false otherwise.
     * @static
     */
    function checkIntervalDates($start, $end, $operationInterval = 0)
    {
        if ($operationInterval < 1) {
            $operationInterval = Maintenance::getOperationInterval();
        }
        if ($operationInterval <= 60) {
            // Must ensure that only one operation interval is being summarised
            $operationIntervalID =
                Maintenance::convertDateRangeToOperationIntervalID($start, $end, $operationInterval);
            if (is_bool($operationIntervalID) && !$operationIntervalID) {
                return false;
            }
            // Now check that the start and end dates match the start and end
            // of the operation interval
            list($operationIntervalStart, $operationIntervalEnd) =
                Maintenance::convertDateToOperationIntervalStartAndEndDates($start, $operationInterval);
            if (!$start->equals($operationIntervalStart)) {
                return false;
            }
            if (!$end->equals($operationIntervalEnd)) {
                return false;
            }
        } else {
            // Must ensure that only one hour is being summarised
            if (!Maintenance::checkDatesInSameHour($start, $end)) {
                return false;
            }
            // Now check that the start and end dates are match the start and
            // end of the hour
            $hourStart = new Date();
            $hourStart->setYear($start->getYear());
            $hourStart->setMonth($start->getMonth());
            $hourStart->setDay($start->getDay());
            $hourStart->setHour($start->getHour());
            $hourStart->setMinute('00');
            $hourStart->setSecond('00');
            $hourEnd = new Date();
            $hourEnd->setYear($end->getYear());
            $hourEnd->setMonth($end->getMonth());
            $hourEnd->setDay($end->getDay());
            $hourEnd->setHour($end->getHour());
            $hourEnd->setMinute('59');
            $hourEnd->setSecond('59');
            if (!$start->equals($hourStart)) {
                return false;
            }
            if (!$end->equals($hourEnd)) {
                return false;
            }
        }
        return true;
    }

   /**
    * A method to return the next Operation Interval(OI) (not current OI)
    * returns an array of PEAR::Date object representing
    * the start (array[0]) and end (array[1]) of the operation interval
    *
    * @return   array containing start and end date same as response from
    *           {@link Maintenance::convertDateToOperationIntervalStartAndEndDates()}
    * @static
    */
    function getNextOperationalInterval($date = '') {
        $date = new Date($date);
        // add operation interval in seconds to date now
        $date->addSeconds(Maintenance::getOperationInterval() * 60);
        return Maintenance::convertDateToOperationIntervalStartAndEndDates($date);
    }

     /**
     * Returns the operation interval value set in global ini file
     *
     * @return integer
     */
    function getOperationInterval() {
        return $GLOBALS['_MAX']['CONF']['maintenance']['operationInterval'];
    }

    /**
     * A method to find the largest, active impression and click connection
     * windows.
     *
     * @static
     * @return array Returns an array containing the largest active impression and
     *               click connection windows. Default values are zero.
     */
    function maxConnectionWindows()
    {
        $oServiceLocator = &ServiceLocator::instance();
        $dbh = &$oServiceLocator->get('MAX_Dal_Maintenance');
        // Set default values
        $impressionWindow = 0;
        $clickWindow = 0;
        // Find the largest, active tracker impression connection window
        $impressionWindow = $dbh->maxAdImpressionWindow();
        // Find the largest, active tracker click connection window
        $clickWindow = $dbh->maxAdClickWindow();
        return array($impressionWindow, $clickWindow);
    }

    /**
     * A static method for premaring e-mails, advising of the activation of
     * campaigns.
     *
     * @static
     * @param string $contactName The name of the campaign contact.
     * @param string $campaignName The name of the deactivated campaign.
     * @param array $ads A reference to an array of ads
     *                              in the campaign, indexed by ad_id,
     *                              of an array containing the description, alt
     *                              description, and  destination URL of the
     *                              ad.
     * @return string The email that has been prepared.
     */
    function prepareActivateCampaignEmail($contactName, $campaignName, &$ads)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $message  = "Dear $contactName,\n\n";
        $message .= 'The following ads have been activated because ' . "\n";
        $message .= 'the campaign activation date has been reached.';
        $message .= "\n\n";
        $message .= "-------------------------------------------------------\n";
        foreach ($ads as $ad_id => $data) {
            $message .= "Ad [ID $ad_id] ";
            if ($data[0] != '') {
                $message .= $data[0];
            } elseif ($data[1] != '') {
                $message .= $data[1];
            } else {
                $message .= 'Untitled';
            }
            $message .= "\n";
            $message .= "Linked to: {$data[2]}\n";
            $message .= "-------------------------------------------------------\n";
        }
        $message .= "\nThank you for advertising with us.\n\n";
        $message .= "Regards,\n\n";
        $message .= $conf['email']['admin_name'];
        return $message;
    }

    /**
     * A static method for preparing e-mails, advising of the deactivation of
     * campaigns.
     *
     * @static
     * @param string $contactName The name of the campaign contact.
     * @param string $campaignName The name of the deactivated campaign.
     * @param integer $reason A binary flag field containting the reason(s) the campaign
     *                        was deactivated:
     *                        2  - No more impressions
     *                        4  - No more clicks
     *                        8  - No more conversions
     *                        16 - Campaign ended (due to date)
     * @param array $ads A reference to an array of ads
     *                              in the campaign, indexed by ad_id,
     *                              of an array containing the description, alt
     *                              description, and  destination URL of the
     *                              ad.
     * @return string The email that has been prepared.
     */
    function prepareDeactivateCampaignEmail($contactName, $campaignName, $reason, &$ads)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $message  = "Dear $contactName,\n\n";
        $message .= 'The following ads have been disabled because:' . "\n";
        if ($reason & MAX_PLACEMENT_DISABLED_IMPRESSIONS) {
            $message .= '  - There are no impressions remaining' . "\n";
        }
        if ($reason & MAX_PLACEMENT_DISABLED_CLICKS) {
            $message .= '  - There are no clicks remaining' . "\n";
        }
        if ($reason & MAX_PLACEMENT_DISABLED_CONVERSIONS) {
            $message .= '  - There are no conversions remaining' . "\n";
        }
        if ($reason & MAX_PLACEMENT_DISABLED_DATE) {
            $message .= '  - The campaign deactivation date has been reached' . "\n";
        }
        $message .= "\n";
        $message .= '-------------------------------------------------------' . "\n";
        foreach ($ads as $ad_id => $data) {
            $message .= "Ad [ID $ad_id] ";
            if ($data[0] != '') {
                $message .= $data[0];
            } elseif ($data[1] != '') {
                $message .= $data[1];
            } else {
                $message .= 'Untitled';
            }
            $message .= "\n";
            $message .= "Linked to: {$data[2]}\n";
            $message .= '-------------------------------------------------------' . "\n";
        }
        $message .= "\n" . 'If you would like to continue advertising on our website,' . "\n";
        $message .= 'please feel free to contact us.' . "\n";
        $message .= 'We\'d be glad to hear from you.' . "\n\n";
        $message .= 'Regards,' . "\n\n";
        $message .= "{$conf['email']['admin_name']}";
        return $message;
    }

    /**
    * Get the number of operation intervals in a given
    * start & end date range must be valid OI start & end date range
    *
    * @param object $oStartDate PEAR::Date object
    * @param object $oEndDate PEAR::Date object
    * @return integer number of operation intervals remaining
    */
    function getIntervalsRemaining($oStartDate, $oEndDate)
    {
        $operationIntervalSeconds = (Maintenance::getOperationInterval() * 60);

        // Get timestamp of start date/time - in seconds
        $startDateSeconds = mktime($oStartDate->getHour(), $oStartDate->getMinute(),
            $oStartDate->getSecond(), $oStartDate->getMonth(),
            $oStartDate->getDay(), $oStartDate->getYear());

        // Get timestamp of end date/time - in seconds
        $endDateSeconds = mktime($oEndDate->getHour(), $oEndDate->getMinute(),
            $oEndDate->getSecond(), $oEndDate->getMonth(),
            $oEndDate->getDay(), $oEndDate->getYear());

        // calculate interval length in seconds
        $interval = $endDateSeconds - $startDateSeconds;

        // find number of operation intervals during interval
        return ($interval <= 0) ? 0 : round($interval / $operationIntervalSeconds);
    }
}

?>
